@@ -2,7 +2,26 @@
 ;; fancy-invoice.scm -- a Fancy Invoice Report, used to print a GncInvoice
 ;;
 ;; Created by:  Derek Atkins <warlord@MIT.EDU>
+;; Copyright (c) 2003 Derek Atkins <warlord@MIT.EDU>
 ;;
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+
+
 ;; Fancy Invoice customized from "invoice.scm"
 ;; Customized by:  Oliver Jones <gnucash at oliverstech dot com>
 ;;
@@ -14,13 +33,18 @@
 ;; Hint: you may set your default options here until a way to save report
 ;; options will be implemented.
 ;;
-;; You will need to upgrade to gtkhtml-1.1 for the latest features or it won't look right.
-;; gtkhtml doesn't have support for table colgroup, tbody, thead and rules
-;; tags yet. When it will, the invoice will look even better.
+;; You will need to upgrade to gtkhtml-1.1 for the latest features or
+;; it won't look right.  gtkhtml doesn't have support for table
+;; colgroup, tbody, thead and rules tags yet. When it will, the
+;; invoice will look even better.
 ;;
-;; This is a quick and dirty hack. The proper way to do this (when I or someone
-;; else will have time) is to have the user supply an HTML template. The most common
-;; used templates will be distributed with gnucash.
+;; This is a quick and dirty hack. The proper way to do this (when I
+;; or someone else will have time) is to have the user supply an HTML
+;; template. The most common used templates will be distributed with
+;; gnucash.
+
+;; Modifed to use settable options instead of the hard coded ones.
+;; modified by Brian Dolbec <dol-sen at telus dot net> Feb. 6, 2006
 
 (define-module (gnucash report fancy-invoice))
 
@@ -29,7 +53,6 @@
 (use-modules (gnucash gnc-module))
 
 (require 'hash-table)
-(require 'record)
 
 (gnc:module-load "gnucash/report/report-system" 0)
 (gnc:module-load "gnucash/business-utils" 0)
@@ -136,7 +159,7 @@
 	    (val (cdr item))
 	    (ref (hash-ref hash acct)))
 
-       (hash-set! hash acct (if ref (gnc:numeric-add-fixed ref val) val))))
+       (hash-set! hash acct (if ref (gnc-numeric-add-fixed ref val) val))))
    values))
 
 (define (monetary-or-percent numeric currency entry-type)
@@ -149,54 +172,54 @@
   (let* ((row-contents '())
 	 (entry-value (gnc:make-gnc-monetary
 		       currency
-		       (gnc:entry-get-value entry invoice?)))
+		       (gncEntryReturnValue entry invoice?)))
 	 (entry-tax-value (gnc:make-gnc-monetary
 			   currency
-			   (gnc:entry-get-tax-value entry invoice?))))
+			   (gncEntryReturnTaxValue entry invoice?))))
 
     (if (date-col column-vector)
         (addto! row-contents
-                (gnc:print-date (gnc:entry-get-date entry))))
+                (gnc-print-date (gncEntryGetDate entry))))
 
     (if (description-col column-vector)
         (addto! row-contents
-		(gnc:entry-get-description entry)))
+		(gncEntryGetDescription entry)))
 
     (if (action-col column-vector)
         (addto! row-contents
-		(gnc:entry-get-action entry)))
+		(gncEntryGetAction entry)))
 
     (if (quantity-col column-vector)
 	(addto! row-contents
 		(gnc:make-html-table-cell/markup
 		 "number-cell"
-		 (gnc:entry-get-quantity entry))))
+		 (gncEntryGetQuantity entry))))
 
     (if (price-col column-vector)
 	(addto! row-contents
 		(gnc:make-html-table-cell/markup
 		 "number-cell"
 		 (gnc:make-gnc-monetary
-		  currency (if invoice? (gnc:entry-get-inv-price entry)
-			       (gnc:entry-get-bill-price entry))))))
+		  currency (if invoice? (gncEntryGetInvPrice entry)
+			       (gncEntryGetBillPrice entry))))))
 
     (if (discount-col column-vector)
 	(addto! row-contents
 		(if invoice?
 		    (gnc:make-html-table-cell/markup
 		     "number-cell"
-		     (monetary-or-percent (gnc:entry-get-inv-discount entry)
+		     (monetary-or-percent (gncEntryGetInvDiscount entry)
 					  currency
-					  (gnc:entry-get-inv-discount-type entry)))
+					  (gncEntryGetInvDiscountType entry)))
 		    "")))
 
     (if (tax-col column-vector)
 	(addto! row-contents
 		(if (if invoice?
-			(and (gnc:entry-get-inv-taxable entry)
-			     (gnc:entry-get-inv-tax-table entry))
-			(and (gnc:entry-get-bill-taxable entry)
-			     (gnc:entry-get-bill-tax-table entry)))
+			(and (gncEntryGetInvTaxable entry)
+			     (gncEntryGetInvTaxTable entry))
+			(and (gncEntryGetBillTaxable entry)
+			     (gncEntryGetBillTaxTable entry)))
 		    (_ "T") "")))
 
     (if (taxvalue-col column-vector)
@@ -227,7 +250,7 @@
 
   (gnc:register-inv-option
    (gnc:make-invoice-option invoice-page invoice-name "x" ""
-			    (lambda () #f) #f))
+			    (lambda () '()) #f))
 
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
@@ -252,7 +275,7 @@
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") (N_ "Price")
-    "hb" "Display the price per item?" #t))
+    "hb" (N_ "Display the price per item?") #t))
 
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
@@ -309,19 +332,50 @@
     (N_ "Display") (N_ "Payments")
     "tc" (N_ "Display the payments applied to this invoice?") #f))
 
+  (gnc:register-inv-option
+   (gnc:make-number-range-option
+    (N_ "Display") (N_ "Minimum # of entries")
+    "u" (N_ "The minimum number of invoice entries to display. (-1)") 23
+    4 23 3 1))
+
   (gnc:register-inv-option
    (gnc:make-text-option
     (N_ "Display") (N_ "Extra Notes")
      "u" (N_ "Extra notes to put on the invoice")
-     ;; oli-custom - Extra notes to add on each invoice, invoice-independent
-     ;; yes, I was too lazy to (get-company-name) ;)
-     "Make all cheques payable to: Company Name Inc.\nDirect all inquiries to: Mr. Accounting Contact"))
+     ""))
+
+  (gnc:register-inv-option
+   (gnc:make-complex-boolean-option
+    (N_ "Display") (N_ "Payable to")
+     "ua1" (N_ "Display the Payable to: information") #t #f
+     (lambda (x) (gnc-option-db-set-option-selectable-by-name
+		  gnc:*report-options* "Display" "Payable to string" x))))
 
   (gnc:register-inv-option
    (gnc:make-string-option
-    (N_ "Display") (N_ "Today Date Format")
-    "v" (N_ "The format for the date->string conversion for today's date.")
-    "%B %e, %Y"))
+    (N_ "Display") (N_ "Payable to string")
+    "ua2" (N_ "The phrase for specifying to whom payments should be made")
+    (_ "Make all cheques Payable to")))
+     
+  (gnc:register-inv-option
+   (gnc:make-complex-boolean-option
+    (N_ "Display") (N_ "Company contact")
+     "ub1" (N_ "Display the Company contact information") #t #f
+     (lambda (x) (gnc-option-db-set-option-selectable-by-name
+		  gnc:*report-options* "Display" "Company contact string" x))))
+
+  (gnc:register-inv-option
+   (gnc:make-string-option
+    (N_ "Display") (N_ "Company contact string")
+    "ub2" (N_ "The phrase used to introduce the company contact")
+    (_ "Direct all inquiries to")))
+
+; not used
+;  (gnc:register-inv-option
+;   (gnc:make-string-option
+;    (N_ "Display") (N_ "Today Date Format")
+;    "v" (N_ "The format for the date->string conversion for today's date.")
+;    (gnc-default-strftime-date-format)))
 
   (gnc:options-set-default-section gnc:*report-options* "General")
 
@@ -334,9 +388,9 @@
 
   (let ((show-payments (opt-val "Display" "Payments"))
 	(display-all-taxes (opt-val "Display" "Individual Taxes"))
-	(lot (gnc:invoice-get-posted-lot invoice))
-	(txn (gnc:invoice-get-posted-txn invoice))
-	(currency (gnc:invoice-get-currency invoice))
+	(lot (gncInvoiceGetPostedLot invoice))
+	(txn (gncInvoiceGetPostedTxn invoice))
+	(currency (gncInvoiceGetCurrency invoice))
 	(entries-added 0))
 
     (define (colspan monetary used-columns)
@@ -350,7 +404,7 @@
 	  monetary
 	  (let ((amt (gnc:gnc-monetary-amount monetary)))
 	    (if amt
-		(if (gnc:numeric-negative-p amt)
+		(if (gnc-numeric-negative-p amt)
 		    (gnc:monetary-neg monetary)
 		    monetary)
 		monetary))))
@@ -364,7 +418,8 @@
 		    (gnc:html-table-append-row/markup! 
 		     table
 		     subtotal-style
-		     ;; oli-custom modified to colspan the subtotal labels instead of the data fields
+		     ;; oli-custom modified to colspan the subtotal labels
+		     ;; instead of the data fields
 		     (append (cons (gnc:make-html-table-cell/size/markup
 				    0 (colspan currency used-columns)
 				    "total-label-cell" subtotal-label)
@@ -376,10 +431,10 @@
 		  currency-totals)))
 
     (define (add-payment-row table used-columns split total-collector)
-      (let* ((t (gnc:split-get-parent split))
-	     (currency (gnc:transaction-get-currency t))
+      (let* ((t (xaccSplitGetParent split))
+	     (currency (xaccTransGetCurrency t))
 	     ;; XXX Need to know when to reverse the value
-	     (amt (gnc:make-gnc-monetary currency (gnc:split-get-value split)))
+	     (amt (gnc:make-gnc-monetary currency (xaccSplitGetValue split)))
 	     (payment-style "grand-total")
 	     (row '()))
 	
@@ -389,7 +444,7 @@
 
 	(if (date-col used-columns)
 	    (addto! row
-		    (gnc:print-date (gnc:transaction-get-date-posted t))))
+		    (gnc-print-date (gnc-transaction-get-date-posted t))))
 
 	(if (description-col used-columns)
 	    (addto! row (_ "Payment, thank you")))
@@ -414,13 +469,16 @@
 				    acct-hash)
       (if (null? entries)
 	  (begin
-	    ;; oli-custom - modified to have a minimum of entries per table, currently at 24
-	    ;; currently doesn't count payment rows and stuff
+	    ;; oli-custom - modified to have a minimum of entries per table,
+	    ;; currently defaults to 24
+	    ;; also, doesn't count payment rows and stuff
 	    (do ((entries-added entries-added (+ entries-added 1))
 		 (odd-row? odd-row? (not odd-row?)))
-		;; oli-custom - here you put the minimum number of rows minus one
-		((> entries-added 23))
-		(gnc:html-table-append-row/markup! table (if odd-row? "normal-row" "alternate-row") (string->list (make-string (num-columns-required used-columns) #\space)))
+		((> entries-added (opt-val "Display" "Minimum # of entries" )))
+		(gnc:html-table-append-row/markup!
+		 table (if odd-row? "normal-row" "alternate-row")
+		 (string->list (make-string (num-columns-required used-columns)
+					    #\space)))
 		)
 	    (add-subtotal-row table used-columns value-collector
 			      "grand-total" (_ "Subtotal"))
@@ -429,33 +487,35 @@
 		(hash-for-each
 		 (lambda (acct value)
 		   (let ((collector (gnc:make-commodity-collector))
-			 (commodity (gnc:account-get-commodity acct))
-			 (name (gnc:account-get-name acct)))
+			 (commodity (xaccAccountGetCommodity acct))
+			 (name (xaccAccountGetName acct)))
 		     (collector 'add commodity value)
 		     (add-subtotal-row table used-columns collector
-				       "grand-total" (string-expand name #\space "&nbsp;"))))
+				       "grand-total" (string-expand
+						      name #\space "&nbsp;"))))
 		 acct-hash)
 
 		; nope, just show the total tax.
 		(add-subtotal-row table used-columns tax-collector
 				  "grand-total" (_ "Tax")))
 
-	    (if (and show-payments lot)
+	    (if (and show-payments (not (null? lot)))
 		(let ((splits (sort-list!
-			       (gnc:lot-get-splits lot)
+			       (gnc-lot-get-split-list lot)
 			       (lambda (s1 s2)
-				 (let ((t1 (gnc:split-get-parent s1))
-				       (t2 (gnc:split-get-parent s2)))
-				   (< (gnc:transaction-order t1 t2) 0))))))
+				 (let ((t1 (xaccSplitGetParent s1))
+				       (t2 (xaccSplitGetParent s2)))
+				   (< (xaccTransOrder t1 t2) 0))))))
 		  (for-each
 		   (lambda (split)
-		     (if (not (equal? (gnc:split-get-parent split) txn))
+		     (if (not (equal? (xaccSplitGetParent split) txn))
 			 (add-payment-row table used-columns
 					  split total-collector)))
 		   splits)))
 
 	    (add-subtotal-row table used-columns total-collector
-			      "grand-total" (string-expand (_ "Amount Due") #\space "&nbsp;")))
+			      "grand-total" (string-expand (_ "Amount Due")
+							   #\space "&nbsp;")))
 
 	  ;;
 	  ;; End of BEGIN -- now here's the code to handle all the entries!
@@ -473,7 +533,7 @@
 					      invoice?)))
 
 	    (if display-all-taxes
-		(let ((tax-list (gnc:entry-get-tax-values current invoice?)))
+		(let ((tax-list (gncEntryReturnTaxValues current invoice?)))
 		  (update-account-hash acct-hash tax-list))
 		(tax-collector 'add
 			       (gnc:gnc-monetary-commodity (cdr entry-values))
@@ -490,8 +550,8 @@
 			     (gnc:gnc-monetary-commodity (cdr entry-values))
 			     (gnc:gnc-monetary-amount (cdr entry-values)))
 
-	    (let ((order (gnc:entry-get-order current)))
-	      (if order (add-order order)))
+	    (let ((order (gncEntryGetOrder current)))
+	      (if (not (null? order)) (add-order order)))
 
 	    (set! entries-added (+ entries-added 1))
 	    
@@ -508,7 +568,7 @@
     (let* ((table (gnc:make-html-table))
 	   (used-columns (build-column-used options))
 	   (width (num-columns-required used-columns))
-	   (entries (gnc:invoice-get-entries invoice))
+	   (entries (gncInvoiceGetEntries invoice))
 	   (totals (gnc:make-commodity-collector)))
 
       (gnc:html-table-set-col-headers!
@@ -545,13 +605,19 @@
   (line-helper (string->list string)))
 
 (define (make-client-table owner orders)
-;; oli-custom - FIXME: font for client company name should be at least size +1.
-  (let ((table (gnc:make-html-table)))
+  (let ((table (gnc:make-html-table))
+	(name-cell (gnc:make-html-table-cell)))
     (gnc:html-table-set-style!
      table "table"
      'attribute (list "border" 0)
      'attribute (list "cellspacing" 0)
      'attribute (list "cellpadding" 0))
+    (gnc:html-table-cell-append-objects!
+     name-cell (gnc:owner-get-name-dep owner))
+    (gnc:html-table-cell-set-style!
+     name-cell "td"
+     'font-size "+2")
+    (gnc:html-table-append-row! table (list name-cell #\newline "<br>"))
     (gnc:html-table-append-row!
      table
      (list
@@ -561,7 +627,7 @@
      (list "<br>"))
     (for-each
      (lambda (order)
-       (let* ((reference (gnc:order-get-reference order)))
+       (let* ((reference (gncOrderGetReference order)))
 	 (if (and reference (> (string-length reference) 0))
 	     (gnc:html-table-append-row!
 	      table
@@ -578,10 +644,13 @@
    table
    (list
     (string-append label ":&nbsp;")
-    ;; oli-custom - modified to display a custom format for the invoice date/due date fields
+    ;; oli-custom - modified to display a custom format
+    ;; for the invoice date/due date fields
     ;; I could have taken the format from the report options, but... ;)
-    (string-expand (strftime "%B %e, %Y" (localtime (car date))) #\space "&nbsp;")
-    ;;(string-expand (gnc:print-date date) #\space "&nbsp;")
+    (string-expand (strftime (gnc-default-strftime-date-format)
+                             (localtime (car date)))
+                   #\space "&nbsp;")
+    ;;(string-expand (gnc-print-date date) #\space "&nbsp;")
     )))
 
 (define (make-date-table)
@@ -597,19 +666,36 @@
 
 (define (make-myname-table book date-format)
   (let* ((table (gnc:make-html-table))
-	 (slots (gnc:book-get-slots book))
-	 (name (gnc:kvp-frame-get-slot-path
+	 (slots (gnc-book-get-slots book))
+	 (name (kvp-frame-get-slot-path-gslist
 		slots (append gnc:*kvp-option-path*
 			      (list gnc:*business-label* gnc:*company-name*))))
-	 (addy (gnc:kvp-frame-get-slot-path
+;;	 (contact (kvp-frame-get-slot-path-gslist
+;;		slots (append gnc:*kvp-option-path*
+;;			      (list gnc:*business-label* gnc:*company-contact*))))
+	 (addy (kvp-frame-get-slot-path-gslist
 		slots (append gnc:*kvp-option-path*
 			      (list gnc:*business-label* gnc:*company-addy*))))
+	 (id (kvp-frame-get-slot-path-gslist
+		slots (append gnc:*kvp-option-path*
+			      (list gnc:*business-label* gnc:*company-id*))))
+	 (phone (kvp-frame-get-slot-path-gslist
+		slots (append gnc:*kvp-option-path*
+			      (list gnc:*business-label* gnc:*company-phone*))))
+	 (fax (kvp-frame-get-slot-path-gslist
+		slots (append gnc:*kvp-option-path*
+			      (list gnc:*business-label* gnc:*company-fax*))))
+	 (url (kvp-frame-get-slot-path-gslist
+		slots (append gnc:*kvp-option-path*
+			      (list gnc:*business-label* gnc:*company-url*))))
 	 (invoice-cell (gnc:make-html-table-cell))
 	 (name-cell (gnc:make-html-table-cell))
 
 	)
-    ;; oli-custom - modified the name table to increase the font size of the company name
-    ;; and add an "INVOICE" title to the upper right, also, put some contact information in the middle
+    ;; oli-custom - modified the name table to increase the
+    ;; font size of the company name
+    ;; and add an "INVOICE" title to the upper right, also,
+    ;; put some contact information in the middle
     ;; FIXME: "INVOICE" should be translated and support bills/expense vouchers
     (gnc:html-table-set-style!
      table "table"
@@ -618,7 +704,7 @@
      'attribute (list "cellpadding" 0)
      'attribute (list "width" "100%"))
     (gnc:html-table-cell-append-objects!
-	invoice-cell "INVOICE")
+	invoice-cell (_"INVOICE"))
     (gnc:html-table-cell-set-style!
 	invoice-cell "td"
 	'font-size "+2")
@@ -636,10 +722,20 @@
 	table 2 "td"
 	'attribute (list "align" "right")
 	'attribute (list "width" "33%"))
-    (gnc:html-table-append-row! table (list (string-expand
-					     (if addy addy "")
-					     #\newline "<br>") "Phone: (111) 222-3333<br>Web: http://companysite.com" ""))
-;; oli-custom - I didn't want today's date on the invoice. The invoice already has a date.
+    (gnc:html-table-append-row!
+     table (list (string-expand (string-append (if addy addy "") (if id (string-append "\n" id) "")) #\newline "<br>")
+		 (string-expand
+		  (string-append (if phone
+				     (string-append (_ "Phone:") " " phone)
+				     "")
+				 (if fax (string-append (if phone "\n" "")
+							(_ "Fax:") " " fax)
+				     ""))
+		  #\newline "<br>" )
+		 (if url (string-append (_ "Web:") " " url) "")))
+
+;; oli-custom - I didn't want today's date on the invoice.
+;; The invoice already has a date.
 ;; Today's date can be in the email, fax or letter accompanying the invoice.
 ;;    (gnc:html-table-append-row! table (list
 ;;				       (strftime
@@ -662,7 +758,7 @@
 	 (table '())
 	 (orders '())
 	 (invoice (opt-val invoice-page invoice-name))
-	 (owner #f)
+	 (owner '())
 	 (references? (opt-val "Display" "References"))
 	 (title (_ "Invoice"))
 	 (invoice? #f))
@@ -671,26 +767,27 @@
       (if (and references? (not (member o orders)))
 	  (addto! orders o)))
 
-    (if invoice
+    (if (not (null? invoice))
 	(begin
-	  (set! owner (gnc:invoice-get-owner invoice))
-	  (let ((type (gw:enum-<gnc:GncOwnerType>-val->sym
-		       (gnc:owner-get-type 
-			(gnc:owner-get-end-owner owner)) #f)))
-	    (case type
-	      ((gnc-owner-customer)
+	  (set! owner (gncInvoiceGetOwner invoice))
+	  (let ((type (gncOwnerGetType
+                       (gncOwnerGetEndOwner owner))))
+	    (cond
+	      ((eqv? type GNC-OWNER-CUSTOMER)
 	       (set! invoice? #t))
-	      ((gnc-owner-vendor)
+	      ((eqv? type GNC-OWNER-VENDOR)
 	       (set! title (_ "Bill")))
-	      ((gnc-owner-employee)
+	      ((eqv? type GNC-OWNER-EMPLOYEE)
 	       (set! title (_ "Expense Voucher")))))
-	  (set! title (string-append title " #"
-				     (gnc:invoice-get-id invoice)))))
-    ;; oli-custom - title redundant, "Invoice" moved to myname-table, invoice number moved below
+	  (set! title (sprintf #f (_"%s #%d") title
+			       (gncInvoiceGetID invoice)))))
+    ;; oli-custom - title redundant, "Invoice" moved to myname-table,
+    ;; invoice number moved below
     ;;(gnc:html-document-set-title! document title)
 
-    (if invoice
-	(let ((book (gnc:invoice-get-book invoice))
+    (if (not (null? invoice))
+	(let* ((book (gncInvoiceGetBook invoice))
+	      (slots (gnc-book-get-slots book))
 	      (date-object #f)
 	      (helper-table (gnc:make-html-table)))
 	  (set! table (make-entry-table invoice
@@ -703,24 +800,33 @@
 	   'attribute (list "cellspacing" 0)
 	   'attribute (list "cellpadding" 4)
 	   ;; oli-custom - make table as wide as possible
-	   ;; works fine with simple style sheet templates, doesn't work quite right with fancy ones
-	   ;; probably supplying the style sheet with a wide image for the header (even if transparent/white) would fix it
+	   ;; works fine with simple style sheet templates,
+	   ;; doesn't work quite right with fancy ones
+	   ;; probably supplying the style sheet with a wide image
+	   ;; for the header (even if transparent/white) would fix it
 	   'attribute (list "width" "100%"))
 
 	  ;; oli-custom - make the description column big
-	  ;; 50% or 60%, depending on whether the first column is displayed or not
-	  ;; should actually be something more complicated, it's a really ugly hack right now :)
+	  ;; 50% or 60%, depending on whether the first column is
+	  ;; displayed or not
+	  ;; should actually be something more complicated,
+	  ;; it's a really ugly hack right now :)
 	  (gnc:html-table-set-col-style!
 	   table (if (opt-val "Display Columns" "Date") 1 0) "td"
-	   'attribute (list "width" (if (opt-val "Display Columns" "Date") "50%" "60%")))
+	   'attribute (list "width" (if (opt-val "Display Columns" "Date")
+					"50%" "60%")))
 
-	  (gnc:html-document-add-object! document (make-myname-table book (opt-val "Display" "Today Date Format")))
+	  (gnc:html-document-add-object!
+	   document (make-myname-table
+		     book ;;(opt-val "Display" "Today Date Format")))
+		     ""))
 
 	  (make-break! document)
 	  (make-break! document)
 	  (make-break! document)
 
-	  ;; oli-custom - client table and table with invoice number/date/due date both inserted into a table
+	  ;; oli-custom - client table and table with invoice
+	  ;; number/date/due date both inserted into a table
 	  (gnc:html-table-set-style!
 	   helper-table "table"
 	   'attribute (list "border" 0)
@@ -729,19 +835,22 @@
 	   'attribute (list "width" "100%"))
 
 	  (set! date-object (let ((date-table #f)
-		(post-date (gnc:invoice-get-date-posted invoice))
-		(due-date (gnc:invoice-get-date-due invoice)))
+		(post-date (gncInvoiceGetDatePosted invoice))
+		(due-date (gncInvoiceGetDateDue invoice)))
 
 	    (if (not (equal? post-date (cons 0 0)))
 		(begin
 		  (set! date-table (make-date-table))
 		  ;; oli-custom - moved invoice number here
-		  (gnc:html-table-append-row! date-table (list "Invoice&nbsp;#&nbsp;" (gnc:invoice-get-id invoice)))
+		  (gnc:html-table-append-row!
+		   date-table (list (sprintf #f (_ "Invoice&nbsp;#&nbsp;%d")
+                                             (gncInvoiceGetID invoice))))
 		  (make-date-row! date-table (_ "Invoice&nbsp;Date") post-date)
 		  (make-date-row! date-table (_ "Due&nbsp;Date") due-date)
 		  date-table)
 		(gnc:make-html-text
-		  ;; oli-custom - FIXME: I have a feeling I broke a translation by not using string-expand for &nbsp;
+		  ;; oli-custom - FIXME: I have a feeling I broke a
+		 ;; translation by not using string-expand for &nbsp;
 		  (string-append title (N_ "<br>Invoice&nbsp;in&nbsp;progress...."))))))
 	  
 	  (gnc:html-table-append-row!
@@ -756,7 +865,8 @@
 	  	helper-table 1 "td"
 		'attribute (list "valign" "top")
 		'attribute (list "align" "right")
-		;; oli-custom - "squeeze" the date table, or else it's spaced out
+		;; oli-custom - "squeeze" the date table,
+		;; or else it's spaced out
 		'attribute (list "width" "1%"))
 
 	  (gnc:html-document-add-object!
@@ -766,7 +876,7 @@
 	  (make-break! document)
 
 	  (if (opt-val "Display" "Billing ID")
-	      (let ((billing-id (gnc:invoice-get-billing-id invoice)))
+	      (let ((billing-id (gncInvoiceGetBillingID invoice)))
 		(if (and billing-id (> (string-length billing-id) 0))
 		    (begin
 		      (gnc:html-document-add-object!
@@ -778,8 +888,8 @@
 		      (make-break! document)))))
 
 	  (if (opt-val "Display" "Billing Terms")
-	      (let* ((term (gnc:invoice-get-terms invoice))
-		     (terms (gnc:bill-term-get-description term)))
+	      (let* ((term (gncInvoiceGetTerms invoice))
+		     (terms (gncBillTermGetDescription term)))
 		(if (and terms (> (string-length terms) 0))
 		    (gnc:html-document-add-object!
 		     document
@@ -796,7 +906,7 @@
 	  (make-break! document)
 
 	  (if (opt-val "Display" "Invoice Notes")
-	      (let ((notes (gnc:invoice-get-notes invoice)))
+	      (let ((notes (gncInvoiceGetNotes invoice)))
 		(gnc:html-document-add-object!
 		 document
 		 (gnc:make-html-text
@@ -804,6 +914,34 @@
 	  
 	  (make-break! document)
 
+	  (if (opt-val "Display" "Payable to")
+	      (let* ((name (kvp-frame-get-slot-path-gslist
+			    slots (append gnc:*kvp-option-path*
+					  (list gnc:*business-label*
+						gnc:*company-name*))))
+		     (name-str (opt-val "Display" "Payable to string")))
+		(if (and name (> (string-length name) 0))
+		(gnc:html-document-add-object!
+		 document
+		 (gnc:make-html-text
+		  (string-append name-str  ":&nbsp;"
+		  (string-expand name #\newline "<br>")))))))
+
+	  (make-break! document)
+
+	  (if (opt-val "Display" "Company contact")
+	      (let* ((contact (kvp-frame-get-slot-path-gslist
+			       slots (append gnc:*kvp-option-path*
+					     (list gnc:*business-label*
+						   gnc:*company-contact*))))
+		     (contact-str (opt-val "Display" "Company contact string")))
+		(if (and contact (> (string-length contact) 0))
+	        (gnc:html-document-add-object!
+		 document
+		 (gnc:make-html-text
+		  (string-append contact-str  ":&nbsp;"
+		  (string-expand contact #\newline "<br>")))))))
+
 	  (gnc:html-document-add-object!
 	   document
 	   (gnc:make-html-text
@@ -815,7 +953,7 @@
 	(gnc:html-document-add-object!
 	 document
 	 (gnc:make-html-text
-	  (N_ "No Valid Invoice Selected"))))
+	  (_ "No valid invoice selected.  Click on the Options button and select the invoice to use."))))
 
     document))
 
